from asdl import const  # For const.NO_INTEGER
from asdl import runtime
from pylib import unpickle

from core import util

f = util.GetResourceLoader().open('_devbuild/syntax_asdl.pickle')
TYPE_LOOKUP = unpickle.load_v2_subset(f)
f.close()

class line_span(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['line_span']
  __slots__ = ('line_id', 'col', 'length', 'spids')

  def __init__(self, line_id=None, col=None, length=None, spids=None):
    self.line_id = line_id
    self.col = col
    self.length = length
    self.spids = spids or []

class token(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['token']
  __slots__ = ('id', 'val', 'span_id', 'spids')

  def __init__(self, id=None, val=None, span_id=None, spids=None):
    self.id = id
    self.val = val
    self.span_id = span_id
    self.spids = spids or []

class braced_step(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['braced_step']
  __slots__ = ('val', 'negated', 'spids')

  def __init__(self, val=None, negated=None, spids=None):
    self.val = val
    self.negated = negated
    self.spids = spids or []

class bracket_op_e(object):
  WholeArray = 1
  ArrayIndex = 2

class bracket_op(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['bracket_op']

class bracket_op__WholeArray(bracket_op):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['bracket_op__WholeArray']
  __slots__ = ('op_id', 'spids')

  def __init__(self, op_id=None, spids=None):
    self.op_id = op_id
    self.spids = spids or []

class bracket_op__ArrayIndex(bracket_op):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['bracket_op__ArrayIndex']
  __slots__ = ('expr', 'spids')

  def __init__(self, expr=None, spids=None):
    self.expr = expr
    self.spids = spids or []

bracket_op.WholeArray = bracket_op__WholeArray
bracket_op.ArrayIndex = bracket_op__ArrayIndex

class suffix_op_e(object):
  StringNullary = 1
  StringUnary = 2
  PatSub = 3
  Slice = 4

class suffix_op(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['suffix_op']

class suffix_op__StringNullary(suffix_op):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['suffix_op__StringNullary']
  __slots__ = ('op_id', 'spids')

  def __init__(self, op_id=None, spids=None):
    self.op_id = op_id
    self.spids = spids or []

class suffix_op__StringUnary(suffix_op):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['suffix_op__StringUnary']
  __slots__ = ('op_id', 'arg_word', 'spids')

  def __init__(self, op_id=None, arg_word=None, spids=None):
    self.op_id = op_id
    self.arg_word = arg_word
    self.spids = spids or []

class suffix_op__PatSub(suffix_op):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['suffix_op__PatSub']
  __slots__ = ('pat', 'replace', 'replace_mode', 'spids')

  def __init__(self, pat=None, replace=None, replace_mode=None, spids=None):
    self.pat = pat
    self.replace = replace or None
    self.replace_mode = replace_mode
    self.spids = spids or []

class suffix_op__Slice(suffix_op):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['suffix_op__Slice']
  __slots__ = ('begin', 'length', 'spids')

  def __init__(self, begin=None, length=None, spids=None):
    self.begin = begin or None
    self.length = length or None
    self.spids = spids or []

suffix_op.StringNullary = suffix_op__StringNullary
suffix_op.StringUnary = suffix_op__StringUnary
suffix_op.PatSub = suffix_op__PatSub
suffix_op.Slice = suffix_op__Slice

class array_item_e(object):
  ArrayWord = 1
  ArrayPair = 2

class array_item(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['array_item']

class array_item__ArrayWord(array_item):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['array_item__ArrayWord']
  __slots__ = ('w', 'spids')

  def __init__(self, w=None, spids=None):
    self.w = w
    self.spids = spids or []

class array_item__ArrayPair(array_item):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['array_item__ArrayPair']
  __slots__ = ('key', 'value', 'spids')

  def __init__(self, key=None, value=None, spids=None):
    self.key = key
    self.value = value
    self.spids = spids or []

array_item.ArrayWord = array_item__ArrayWord
array_item.ArrayPair = array_item__ArrayPair

class word_part_e(object):
  ArrayLiteralPart = 1
  LiteralPart = 2
  EscapedLiteralPart = 3
  SingleQuotedPart = 4
  DoubleQuotedPart = 5
  SimpleVarSub = 6
  BracedVarSub = 7
  TildeSubPart = 8
  CommandSubPart = 9
  ArithSubPart = 10
  BracedAltPart = 11
  BracedIntRangePart = 12
  BracedCharRangePart = 13
  ExtGlobPart = 14

class word_part(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['word_part']

class word_part__ArrayLiteralPart(word_part):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['word_part__ArrayLiteralPart']
  __slots__ = ('words', 'spids')

  def __init__(self, words=None, spids=None):
    self.words = words or []
    self.spids = spids or []

class word_part__LiteralPart(word_part):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['word_part__LiteralPart']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class word_part__EscapedLiteralPart(word_part):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['word_part__EscapedLiteralPart']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class word_part__SingleQuotedPart(word_part):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['word_part__SingleQuotedPart']
  __slots__ = ('left', 'tokens', 'spids')

  def __init__(self, left=None, tokens=None, spids=None):
    self.left = left
    self.tokens = tokens or []
    self.spids = spids or []

class word_part__DoubleQuotedPart(word_part):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['word_part__DoubleQuotedPart']
  __slots__ = ('parts', 'spids')

  def __init__(self, parts=None, spids=None):
    self.parts = parts or []
    self.spids = spids or []

class word_part__SimpleVarSub(word_part):
  tag = 6
  ASDL_TYPE = TYPE_LOOKUP['word_part__SimpleVarSub']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class word_part__BracedVarSub(word_part):
  tag = 7
  ASDL_TYPE = TYPE_LOOKUP['word_part__BracedVarSub']
  __slots__ = ('token', 'prefix_op', 'bracket_op', 'suffix_op', 'spids')

  def __init__(self, token=None, prefix_op=None, bracket_op=None,
               suffix_op=None, spids=None):
    self.token = token
    self.prefix_op = prefix_op or None
    self.bracket_op = bracket_op or None
    self.suffix_op = suffix_op or None
    self.spids = spids or []

class word_part__TildeSubPart(word_part):
  tag = 8
  ASDL_TYPE = TYPE_LOOKUP['word_part__TildeSubPart']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class word_part__CommandSubPart(word_part):
  tag = 9
  ASDL_TYPE = TYPE_LOOKUP['word_part__CommandSubPart']
  __slots__ = ('command_list', 'left_token', 'spids')

  def __init__(self, command_list=None, left_token=None, spids=None):
    self.command_list = command_list
    self.left_token = left_token
    self.spids = spids or []

class word_part__ArithSubPart(word_part):
  tag = 10
  ASDL_TYPE = TYPE_LOOKUP['word_part__ArithSubPart']
  __slots__ = ('anode', 'spids')

  def __init__(self, anode=None, spids=None):
    self.anode = anode
    self.spids = spids or []

class word_part__BracedAltPart(word_part):
  tag = 11
  ASDL_TYPE = TYPE_LOOKUP['word_part__BracedAltPart']
  __slots__ = ('words', 'spids')

  def __init__(self, words=None, spids=None):
    self.words = words or []
    self.spids = spids or []

class word_part__BracedIntRangePart(word_part):
  tag = 12
  ASDL_TYPE = TYPE_LOOKUP['word_part__BracedIntRangePart']
  __slots__ = ('start', 'end', 'step', 'spids')

  def __init__(self, start=None, end=None, step=None, spids=None):
    self.start = start
    self.end = end
    self.step = step or None
    self.spids = spids or []

class word_part__BracedCharRangePart(word_part):
  tag = 13
  ASDL_TYPE = TYPE_LOOKUP['word_part__BracedCharRangePart']
  __slots__ = ('start', 'end', 'step', 'spids')

  def __init__(self, start=None, end=None, step=None, spids=None):
    self.start = start
    self.end = end
    self.step = step or None
    self.spids = spids or []

class word_part__ExtGlobPart(word_part):
  tag = 14
  ASDL_TYPE = TYPE_LOOKUP['word_part__ExtGlobPart']
  __slots__ = ('op', 'arms', 'spids')

  def __init__(self, op=None, arms=None, spids=None):
    self.op = op
    self.arms = arms or []
    self.spids = spids or []

word_part.ArrayLiteralPart = word_part__ArrayLiteralPart
word_part.LiteralPart = word_part__LiteralPart
word_part.EscapedLiteralPart = word_part__EscapedLiteralPart
word_part.SingleQuotedPart = word_part__SingleQuotedPart
word_part.DoubleQuotedPart = word_part__DoubleQuotedPart
word_part.SimpleVarSub = word_part__SimpleVarSub
word_part.BracedVarSub = word_part__BracedVarSub
word_part.TildeSubPart = word_part__TildeSubPart
word_part.CommandSubPart = word_part__CommandSubPart
word_part.ArithSubPart = word_part__ArithSubPart
word_part.BracedAltPart = word_part__BracedAltPart
word_part.BracedIntRangePart = word_part__BracedIntRangePart
word_part.BracedCharRangePart = word_part__BracedCharRangePart
word_part.ExtGlobPart = word_part__ExtGlobPart

class word_e(object):
  EmptyWord = 1
  TokenWord = 2
  CompoundWord = 3
  BracedWordTree = 4
  StringWord = 5

class word(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['word']

class word__EmptyWord(word):
  ASDL_TYPE = TYPE_LOOKUP['word__EmptyWord']
  tag = 1

class word__TokenWord(word):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['word__TokenWord']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class word__CompoundWord(word):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['word__CompoundWord']
  __slots__ = ('parts', 'spids')

  def __init__(self, parts=None, spids=None):
    self.parts = parts or []
    self.spids = spids or []

class word__BracedWordTree(word):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['word__BracedWordTree']
  __slots__ = ('parts', 'spids')

  def __init__(self, parts=None, spids=None):
    self.parts = parts or []
    self.spids = spids or []

class word__StringWord(word):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['word__StringWord']
  __slots__ = ('id', 's', 'spids')

  def __init__(self, id=None, s=None, spids=None):
    self.id = id
    self.s = s
    self.spids = spids or []

word.EmptyWord = word__EmptyWord
word.TokenWord = word__TokenWord
word.CompoundWord = word__CompoundWord
word.BracedWordTree = word__BracedWordTree
word.StringWord = word__StringWord

class lhs_expr_e(object):
  LhsName = 1
  LhsIndexedName = 2
  CompatIndexedName = 3

class lhs_expr(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['lhs_expr']

class lhs_expr__LhsName(lhs_expr):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['lhs_expr__LhsName']
  __slots__ = ('name', 'spids')

  def __init__(self, name=None, spids=None):
    self.name = name
    self.spids = spids or []

class lhs_expr__LhsIndexedName(lhs_expr):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['lhs_expr__LhsIndexedName']
  __slots__ = ('name', 'index', 'spids')

  def __init__(self, name=None, index=None, spids=None):
    self.name = name
    self.index = index
    self.spids = spids or []

class lhs_expr__CompatIndexedName(lhs_expr):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['lhs_expr__CompatIndexedName']
  __slots__ = ('name', 'index', 'spids')

  def __init__(self, name=None, index=None, spids=None):
    self.name = name
    self.index = index
    self.spids = spids or []

lhs_expr.LhsName = lhs_expr__LhsName
lhs_expr.LhsIndexedName = lhs_expr__LhsIndexedName
lhs_expr.CompatIndexedName = lhs_expr__CompatIndexedName

class arith_expr_e(object):
  ArithVarRef = 1
  ArithWord = 2
  UnaryAssign = 3
  BinaryAssign = 4
  ArithUnary = 5
  ArithBinary = 6
  TernaryOp = 7
  FuncCall = 8

class arith_expr(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['arith_expr']

class arith_expr__ArithVarRef(arith_expr):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__ArithVarRef']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class arith_expr__ArithWord(arith_expr):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__ArithWord']
  __slots__ = ('w', 'spids')

  def __init__(self, w=None, spids=None):
    self.w = w
    self.spids = spids or []

class arith_expr__UnaryAssign(arith_expr):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__UnaryAssign']
  __slots__ = ('op_id', 'child', 'spids')

  def __init__(self, op_id=None, child=None, spids=None):
    self.op_id = op_id
    self.child = child
    self.spids = spids or []

class arith_expr__BinaryAssign(arith_expr):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__BinaryAssign']
  __slots__ = ('op_id', 'left', 'right', 'spids')

  def __init__(self, op_id=None, left=None, right=None, spids=None):
    self.op_id = op_id
    self.left = left
    self.right = right
    self.spids = spids or []

class arith_expr__ArithUnary(arith_expr):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__ArithUnary']
  __slots__ = ('op_id', 'child', 'spids')

  def __init__(self, op_id=None, child=None, spids=None):
    self.op_id = op_id
    self.child = child
    self.spids = spids or []

class arith_expr__ArithBinary(arith_expr):
  tag = 6
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__ArithBinary']
  __slots__ = ('op_id', 'left', 'right', 'spids')

  def __init__(self, op_id=None, left=None, right=None, spids=None):
    self.op_id = op_id
    self.left = left
    self.right = right
    self.spids = spids or []

class arith_expr__TernaryOp(arith_expr):
  tag = 7
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__TernaryOp']
  __slots__ = ('cond', 'true_expr', 'false_expr', 'spids')

  def __init__(self, cond=None, true_expr=None, false_expr=None, spids=None):
    self.cond = cond
    self.true_expr = true_expr
    self.false_expr = false_expr
    self.spids = spids or []

class arith_expr__FuncCall(arith_expr):
  tag = 8
  ASDL_TYPE = TYPE_LOOKUP['arith_expr__FuncCall']
  __slots__ = ('func', 'args', 'spids')

  def __init__(self, func=None, args=None, spids=None):
    self.func = func
    self.args = args or []
    self.spids = spids or []

arith_expr.ArithVarRef = arith_expr__ArithVarRef
arith_expr.ArithWord = arith_expr__ArithWord
arith_expr.UnaryAssign = arith_expr__UnaryAssign
arith_expr.BinaryAssign = arith_expr__BinaryAssign
arith_expr.ArithUnary = arith_expr__ArithUnary
arith_expr.ArithBinary = arith_expr__ArithBinary
arith_expr.TernaryOp = arith_expr__TernaryOp
arith_expr.FuncCall = arith_expr__FuncCall

class bool_expr_e(object):
  WordTest = 1
  BoolBinary = 2
  BoolUnary = 3
  LogicalNot = 4
  LogicalAnd = 5
  LogicalOr = 6

class bool_expr(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['bool_expr']

class bool_expr__WordTest(bool_expr):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['bool_expr__WordTest']
  __slots__ = ('w', 'spids')

  def __init__(self, w=None, spids=None):
    self.w = w
    self.spids = spids or []

class bool_expr__BoolBinary(bool_expr):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['bool_expr__BoolBinary']
  __slots__ = ('op_id', 'left', 'right', 'spids')

  def __init__(self, op_id=None, left=None, right=None, spids=None):
    self.op_id = op_id
    self.left = left
    self.right = right
    self.spids = spids or []

class bool_expr__BoolUnary(bool_expr):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['bool_expr__BoolUnary']
  __slots__ = ('op_id', 'child', 'spids')

  def __init__(self, op_id=None, child=None, spids=None):
    self.op_id = op_id
    self.child = child
    self.spids = spids or []

class bool_expr__LogicalNot(bool_expr):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['bool_expr__LogicalNot']
  __slots__ = ('child', 'spids')

  def __init__(self, child=None, spids=None):
    self.child = child
    self.spids = spids or []

class bool_expr__LogicalAnd(bool_expr):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['bool_expr__LogicalAnd']
  __slots__ = ('left', 'right', 'spids')

  def __init__(self, left=None, right=None, spids=None):
    self.left = left
    self.right = right
    self.spids = spids or []

class bool_expr__LogicalOr(bool_expr):
  tag = 6
  ASDL_TYPE = TYPE_LOOKUP['bool_expr__LogicalOr']
  __slots__ = ('left', 'right', 'spids')

  def __init__(self, left=None, right=None, spids=None):
    self.left = left
    self.right = right
    self.spids = spids or []

bool_expr.WordTest = bool_expr__WordTest
bool_expr.BoolBinary = bool_expr__BoolBinary
bool_expr.BoolUnary = bool_expr__BoolUnary
bool_expr.LogicalNot = bool_expr__LogicalNot
bool_expr.LogicalAnd = bool_expr__LogicalAnd
bool_expr.LogicalOr = bool_expr__LogicalOr

class redir_e(object):
  Redir = 1
  HereDoc = 2

class redir(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['redir']

class redir__Redir(redir):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['redir__Redir']
  __slots__ = ('op', 'fd', 'arg_word', 'spids')

  def __init__(self, op=None, fd=None, arg_word=None, spids=None):
    self.op = op
    self.fd = fd
    self.arg_word = arg_word
    self.spids = spids or []

class redir__HereDoc(redir):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['redir__HereDoc']
  __slots__ = ('op', 'fd', 'here_begin', 'here_end_span_id', 'stdin_parts',
               'spids')

  def __init__(self, op=None, fd=None, here_begin=None, here_end_span_id=None,
               stdin_parts=None, spids=None):
    self.op = op
    self.fd = fd
    self.here_begin = here_begin
    self.here_end_span_id = here_end_span_id
    self.stdin_parts = stdin_parts or []
    self.spids = spids or []

redir.Redir = redir__Redir
redir.HereDoc = redir__HereDoc

class assign_op_e(runtime.SimpleObj):
  ASDL_TYPE = TYPE_LOOKUP['assign_op']

assign_op_e.Equal = assign_op_e(1, 'Equal')
assign_op_e.PlusEqual = assign_op_e(2, 'PlusEqual')

class assign_pair(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['assign_pair']
  __slots__ = ('lhs', 'op', 'rhs', 'spids')

  def __init__(self, lhs=None, op=None, rhs=None, spids=None):
    self.lhs = lhs
    self.op = op
    self.rhs = rhs or None
    self.spids = spids or []

class env_pair(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['env_pair']
  __slots__ = ('name', 'val', 'spids')

  def __init__(self, name=None, val=None, spids=None):
    self.name = name
    self.val = val
    self.spids = spids or []

class case_arm(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['case_arm']
  __slots__ = ('pat_list', 'action', 'spids')

  def __init__(self, pat_list=None, action=None, spids=None):
    self.pat_list = pat_list or []
    self.action = action or []
    self.spids = spids or []

class if_arm(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['if_arm']
  __slots__ = ('cond', 'action', 'spids')

  def __init__(self, cond=None, action=None, spids=None):
    self.cond = cond or []
    self.action = action or []
    self.spids = spids or []

class iterable_e(object):
  IterArgv = 1
  IterArray = 2

class iterable(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['iterable']

class iterable__IterArgv(iterable):
  ASDL_TYPE = TYPE_LOOKUP['iterable__IterArgv']
  tag = 1

class iterable__IterArray(iterable):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['iterable__IterArray']
  __slots__ = ('words', 'spids')

  def __init__(self, words=None, spids=None):
    self.words = words or []
    self.spids = spids or []

iterable.IterArgv = iterable__IterArgv
iterable.IterArray = iterable__IterArray

class command_e(object):
  NoOp = 1
  SimpleCommand = 2
  Sentence = 3
  Assignment = 4
  ControlFlow = 5
  Pipeline = 6
  AndOr = 7
  DoGroup = 8
  BraceGroup = 9
  Subshell = 10
  DParen = 11
  DBracket = 12
  ForEach = 13
  ForExpr = 14
  WhileUntil = 15
  If = 16
  Case = 17
  FuncDef = 18
  TimeBlock = 19
  CommandList = 20

class command(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['command']

class command__NoOp(command):
  ASDL_TYPE = TYPE_LOOKUP['command__NoOp']
  tag = 1

class command__SimpleCommand(command):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['command__SimpleCommand']
  __slots__ = ('words', 'redirects', 'more_env', 'spids')

  def __init__(self, words=None, redirects=None, more_env=None, spids=None):
    self.words = words or []
    self.redirects = redirects or []
    self.more_env = more_env or []
    self.spids = spids or []

class command__Sentence(command):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['command__Sentence']
  __slots__ = ('child', 'terminator', 'spids')

  def __init__(self, child=None, terminator=None, spids=None):
    self.child = child
    self.terminator = terminator
    self.spids = spids or []

class command__Assignment(command):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['command__Assignment']
  __slots__ = ('keyword', 'flags', 'pairs', 'spids')

  def __init__(self, keyword=None, flags=None, pairs=None, spids=None):
    self.keyword = keyword
    self.flags = flags or []
    self.pairs = pairs or []
    self.spids = spids or []

class command__ControlFlow(command):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['command__ControlFlow']
  __slots__ = ('token', 'arg_word', 'spids')

  def __init__(self, token=None, arg_word=None, spids=None):
    self.token = token
    self.arg_word = arg_word or None
    self.spids = spids or []

class command__Pipeline(command):
  tag = 6
  ASDL_TYPE = TYPE_LOOKUP['command__Pipeline']
  __slots__ = ('children', 'negated', 'stderr_indices', 'spids')

  def __init__(self, children=None, negated=None, stderr_indices=None,
               spids=None):
    self.children = children or []
    self.negated = negated
    self.stderr_indices = stderr_indices or []
    self.spids = spids or []

class command__AndOr(command):
  tag = 7
  ASDL_TYPE = TYPE_LOOKUP['command__AndOr']
  __slots__ = ('ops', 'children', 'spids')

  def __init__(self, ops=None, children=None, spids=None):
    self.ops = ops or []
    self.children = children or []
    self.spids = spids or []

class command__DoGroup(command):
  tag = 8
  ASDL_TYPE = TYPE_LOOKUP['command__DoGroup']
  __slots__ = ('children', 'redirects', 'spids')

  def __init__(self, children=None, redirects=None, spids=None):
    self.children = children or []
    self.redirects = redirects or []
    self.spids = spids or []

class command__BraceGroup(command):
  tag = 9
  ASDL_TYPE = TYPE_LOOKUP['command__BraceGroup']
  __slots__ = ('children', 'redirects', 'spids')

  def __init__(self, children=None, redirects=None, spids=None):
    self.children = children or []
    self.redirects = redirects or []
    self.spids = spids or []

class command__Subshell(command):
  tag = 10
  ASDL_TYPE = TYPE_LOOKUP['command__Subshell']
  __slots__ = ('child', 'redirects', 'spids')

  def __init__(self, child=None, redirects=None, spids=None):
    self.child = child
    self.redirects = redirects or []
    self.spids = spids or []

class command__DParen(command):
  tag = 11
  ASDL_TYPE = TYPE_LOOKUP['command__DParen']
  __slots__ = ('child', 'redirects', 'spids')

  def __init__(self, child=None, redirects=None, spids=None):
    self.child = child
    self.redirects = redirects or []
    self.spids = spids or []

class command__DBracket(command):
  tag = 12
  ASDL_TYPE = TYPE_LOOKUP['command__DBracket']
  __slots__ = ('expr', 'redirects', 'spids')

  def __init__(self, expr=None, redirects=None, spids=None):
    self.expr = expr
    self.redirects = redirects or []
    self.spids = spids or []

class command__ForEach(command):
  tag = 13
  ASDL_TYPE = TYPE_LOOKUP['command__ForEach']
  __slots__ = ('iter_name', 'iter_words', 'do_arg_iter', 'body', 'redirects',
               'spids')

  def __init__(self, iter_name=None, iter_words=None, do_arg_iter=None,
               body=None, redirects=None, spids=None):
    self.iter_name = iter_name
    self.iter_words = iter_words or []
    self.do_arg_iter = do_arg_iter
    self.body = body
    self.redirects = redirects or []
    self.spids = spids or []

class command__ForExpr(command):
  tag = 14
  ASDL_TYPE = TYPE_LOOKUP['command__ForExpr']
  __slots__ = ('init', 'cond', 'update', 'body', 'redirects', 'spids')

  def __init__(self, init=None, cond=None, update=None, body=None,
               redirects=None, spids=None):
    self.init = init or None
    self.cond = cond or None
    self.update = update or None
    self.body = body or None
    self.redirects = redirects or []
    self.spids = spids or []

class command__WhileUntil(command):
  tag = 15
  ASDL_TYPE = TYPE_LOOKUP['command__WhileUntil']
  __slots__ = ('keyword', 'cond', 'body', 'redirects', 'spids')

  def __init__(self, keyword=None, cond=None, body=None, redirects=None,
               spids=None):
    self.keyword = keyword
    self.cond = cond or []
    self.body = body
    self.redirects = redirects or []
    self.spids = spids or []

class command__If(command):
  tag = 16
  ASDL_TYPE = TYPE_LOOKUP['command__If']
  __slots__ = ('arms', 'else_action', 'redirects', 'spids')

  def __init__(self, arms=None, else_action=None, redirects=None, spids=None):
    self.arms = arms or []
    self.else_action = else_action or []
    self.redirects = redirects or []
    self.spids = spids or []

class command__Case(command):
  tag = 17
  ASDL_TYPE = TYPE_LOOKUP['command__Case']
  __slots__ = ('to_match', 'arms', 'redirects', 'spids')

  def __init__(self, to_match=None, arms=None, redirects=None, spids=None):
    self.to_match = to_match
    self.arms = arms or []
    self.redirects = redirects or []
    self.spids = spids or []

class command__FuncDef(command):
  tag = 18
  ASDL_TYPE = TYPE_LOOKUP['command__FuncDef']
  __slots__ = ('name', 'body', 'redirects', 'spids')

  def __init__(self, name=None, body=None, redirects=None, spids=None):
    self.name = name
    self.body = body
    self.redirects = redirects or []
    self.spids = spids or []

class command__TimeBlock(command):
  tag = 19
  ASDL_TYPE = TYPE_LOOKUP['command__TimeBlock']
  __slots__ = ('pipeline', 'spids')

  def __init__(self, pipeline=None, spids=None):
    self.pipeline = pipeline
    self.spids = spids or []

class command__CommandList(command):
  tag = 20
  ASDL_TYPE = TYPE_LOOKUP['command__CommandList']
  __slots__ = ('children', 'spids')

  def __init__(self, children=None, spids=None):
    self.children = children or []
    self.spids = spids or []

command.NoOp = command__NoOp
command.SimpleCommand = command__SimpleCommand
command.Sentence = command__Sentence
command.Assignment = command__Assignment
command.ControlFlow = command__ControlFlow
command.Pipeline = command__Pipeline
command.AndOr = command__AndOr
command.DoGroup = command__DoGroup
command.BraceGroup = command__BraceGroup
command.Subshell = command__Subshell
command.DParen = command__DParen
command.DBracket = command__DBracket
command.ForEach = command__ForEach
command.ForExpr = command__ForExpr
command.WhileUntil = command__WhileUntil
command.If = command__If
command.Case = command__Case
command.FuncDef = command__FuncDef
command.TimeBlock = command__TimeBlock
command.CommandList = command__CommandList

class glob_part_e(object):
  GlobLit = 1
  GlobOp = 2
  CharClass = 3

class glob_part(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['glob_part']

class glob_part__GlobLit(glob_part):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['glob_part__GlobLit']
  __slots__ = ('id', 's', 'spids')

  def __init__(self, id=None, s=None, spids=None):
    self.id = id
    self.s = s
    self.spids = spids or []

class glob_part__GlobOp(glob_part):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['glob_part__GlobOp']
  __slots__ = ('op_id', 'spids')

  def __init__(self, op_id=None, spids=None):
    self.op_id = op_id
    self.spids = spids or []

class glob_part__CharClass(glob_part):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['glob_part__CharClass']
  __slots__ = ('negated', 'strs', 'spids')

  def __init__(self, negated=None, strs=None, spids=None):
    self.negated = negated
    self.strs = strs or []
    self.spids = spids or []

glob_part.GlobLit = glob_part__GlobLit
glob_part.GlobOp = glob_part__GlobOp
glob_part.CharClass = glob_part__CharClass

class oil_expr_e(object):
  Name = 1
  UnaryOp = 2
  BinaryOp = 3
  FuncCall = 4

class oil_expr(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['oil_expr']

class oil_expr__Name(oil_expr):
  ASDL_TYPE = TYPE_LOOKUP['oil_expr__Name']
  tag = 1

class oil_expr__UnaryOp(oil_expr):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['oil_expr__UnaryOp']
  __slots__ = ('op', 'child', 'spids')

  def __init__(self, op=None, child=None, spids=None):
    self.op = op
    self.child = child
    self.spids = spids or []

class oil_expr__BinaryOp(oil_expr):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['oil_expr__BinaryOp']
  __slots__ = ('op', 'left', 'right', 'spids')

  def __init__(self, op=None, left=None, right=None, spids=None):
    self.op = op
    self.left = left
    self.right = right
    self.spids = spids or []

class oil_expr__FuncCall(oil_expr):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['oil_expr__FuncCall']
  __slots__ = ('func', 'args', 'spids')

  def __init__(self, func=None, args=None, spids=None):
    self.func = func
    self.args = args or []
    self.spids = spids or []

oil_expr.Name = oil_expr__Name
oil_expr.UnaryOp = oil_expr__UnaryOp
oil_expr.BinaryOp = oil_expr__BinaryOp
oil_expr.FuncCall = oil_expr__FuncCall

class oil_word_part_e(object):
  Literal = 1
  EscapedLiteral = 2
  SingleQuoted = 3
  DoubleQuoted = 4
  TildeSub = 5
  VarSub = 6
  ExprSub = 7
  CommandSub = 8

class oil_word_part(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part']

class oil_word_part__Literal(oil_word_part):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__Literal']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class oil_word_part__EscapedLiteral(oil_word_part):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__EscapedLiteral']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class oil_word_part__SingleQuoted(oil_word_part):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__SingleQuoted']
  __slots__ = ('left', 'tokens', 'spids')

  def __init__(self, left=None, tokens=None, spids=None):
    self.left = left
    self.tokens = tokens or []
    self.spids = spids or []

class oil_word_part__DoubleQuoted(oil_word_part):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__DoubleQuoted']
  __slots__ = ('left', 'parts', 'spids')

  def __init__(self, left=None, parts=None, spids=None):
    self.left = left
    self.parts = parts or []
    self.spids = spids or []

class oil_word_part__TildeSub(oil_word_part):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__TildeSub']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class oil_word_part__VarSub(oil_word_part):
  tag = 6
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__VarSub']
  __slots__ = ('token', 'spids')

  def __init__(self, token=None, spids=None):
    self.token = token
    self.spids = spids or []

class oil_word_part__ExprSub(oil_word_part):
  tag = 7
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__ExprSub']
  __slots__ = ('token', 'prefix_op', 'bracket_op', 'suffix_op', 'spids')

  def __init__(self, token=None, prefix_op=None, bracket_op=None,
               suffix_op=None, spids=None):
    self.token = token
    self.prefix_op = prefix_op or None
    self.bracket_op = bracket_op or None
    self.suffix_op = suffix_op or None
    self.spids = spids or []

class oil_word_part__CommandSub(oil_word_part):
  tag = 8
  ASDL_TYPE = TYPE_LOOKUP['oil_word_part__CommandSub']
  __slots__ = ('command_list', 'left_token', 'spids')

  def __init__(self, command_list=None, left_token=None, spids=None):
    self.command_list = command_list
    self.left_token = left_token
    self.spids = spids or []

oil_word_part.Literal = oil_word_part__Literal
oil_word_part.EscapedLiteral = oil_word_part__EscapedLiteral
oil_word_part.SingleQuoted = oil_word_part__SingleQuoted
oil_word_part.DoubleQuoted = oil_word_part__DoubleQuoted
oil_word_part.TildeSub = oil_word_part__TildeSub
oil_word_part.VarSub = oil_word_part__VarSub
oil_word_part.ExprSub = oil_word_part__ExprSub
oil_word_part.CommandSub = oil_word_part__CommandSub

class oil_word_e(object):
  Compound = 1
  Glob = 2
  Braced = 3

class oil_word(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['oil_word']

class oil_word__Compound(oil_word):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['oil_word__Compound']
  __slots__ = ('parts', 'spids')

  def __init__(self, parts=None, spids=None):
    self.parts = parts or []
    self.spids = spids or []

class oil_word__Glob(oil_word):
  ASDL_TYPE = TYPE_LOOKUP['oil_word__Glob']
  tag = 2

class oil_word__Braced(oil_word):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['oil_word__Braced']
  __slots__ = ('parts', 'spids')

  def __init__(self, parts=None, spids=None):
    self.parts = parts or []
    self.spids = spids or []

oil_word.Compound = oil_word__Compound
oil_word.Glob = oil_word__Glob
oil_word.Braced = oil_word__Braced

class oil_cmd_e(object):
  Simple = 1
  Sentence = 2
  Pipeline = 3
  AndOr = 4
  CommandList = 5

class oil_cmd(runtime.CompoundObj):
  ASDL_TYPE = TYPE_LOOKUP['oil_cmd']

class oil_cmd__Simple(oil_cmd):
  tag = 1
  ASDL_TYPE = TYPE_LOOKUP['oil_cmd__Simple']
  __slots__ = ('words', 'redirects', 'more_env', 'spids')

  def __init__(self, words=None, redirects=None, more_env=None, spids=None):
    self.words = words or []
    self.redirects = redirects or []
    self.more_env = more_env or []
    self.spids = spids or []

class oil_cmd__Sentence(oil_cmd):
  tag = 2
  ASDL_TYPE = TYPE_LOOKUP['oil_cmd__Sentence']
  __slots__ = ('child', 'terminator', 'spids')

  def __init__(self, child=None, terminator=None, spids=None):
    self.child = child
    self.terminator = terminator
    self.spids = spids or []

class oil_cmd__Pipeline(oil_cmd):
  tag = 3
  ASDL_TYPE = TYPE_LOOKUP['oil_cmd__Pipeline']
  __slots__ = ('children', 'negated', 'spids')

  def __init__(self, children=None, negated=None, spids=None):
    self.children = children or []
    self.negated = negated
    self.spids = spids or []

class oil_cmd__AndOr(oil_cmd):
  tag = 4
  ASDL_TYPE = TYPE_LOOKUP['oil_cmd__AndOr']
  __slots__ = ('ops', 'children', 'spids')

  def __init__(self, ops=None, children=None, spids=None):
    self.ops = ops or []
    self.children = children or []
    self.spids = spids or []

class oil_cmd__CommandList(oil_cmd):
  tag = 5
  ASDL_TYPE = TYPE_LOOKUP['oil_cmd__CommandList']
  __slots__ = ('children', 'spids')

  def __init__(self, children=None, spids=None):
    self.children = children or []
    self.spids = spids or []

oil_cmd.Simple = oil_cmd__Simple
oil_cmd.Sentence = oil_cmd__Sentence
oil_cmd.Pipeline = oil_cmd__Pipeline
oil_cmd.AndOr = oil_cmd__AndOr
oil_cmd.CommandList = oil_cmd__CommandList

